; This procedure loads the proper TOLNet-compliant format template
; For questions/comments, contact Thierry Leblanc at thierry.leblanc@jpl.nasa.gov
; but... I will not answer your questions if you do not know IDL programming

pro TOLNetFormat,version ,ngenheadformat,ngencommformat,ngencommmeta,nrecformat,nrecmeta,genheadlabel,genhead,gencommlabel,reclabel,datalabel,missval,ioformat

; Inputs:
;    "version" (scalar string) = TOLNet format version # (default is v1.0)

; Outputs:
;    "ngenheadformat" (scalar integer) = Number of lines in general header section  dedicated to file format defintion and structure (fixed for a given TOLNet format)
;    "ngencommformat" (scalar integer) = Number of lines in general comment section dedicated to file format defintion and structure (fixed for ALL TOLNet formats)
;    "ngencommmeta"   (scalar integer) = Number of lines in general comment section dedicated to meta data (fixed for a given TOLNet format)
;    "nrecformat"     (scalar integer) = Number of lines in record header dedicated to file format defintion and structure (fixed for ALL TOLNet formats)
;    "nrecmeta"       (scalar integer) = Number of lines in record header dedicated to meta data (fixed for a given TOLNet format)
;    "genheadlabel"   (vector string)  = Content of general header section labels (right hand side of separator " ; ")
;    "genhead"        (vector string)  = Content of general header section meta data (left hand side of separator " ; ")
;    "gencommlabel"   (vector string)  = Content of general comment section labels (right hand side of separator " ; ")
;    "reclabel"       (vector string)  = Content of record header labels (right hand side of separator " ; ")
;    "datalabel"      (vector string)  = Content of data columns header (just above data section)
;    "missval"        (scalar string)  = Content of missing values

; Programming history
;-------------------------------------------------------------------------------------------------------------
; 2012-12-04: Subroutine created
; 2012-12-04: TOLNet profile data format version 1.0 as of 29-Nov-2012: 5 general header metadata lines, 10 profile header metadata lines, 14 data columns
;---------------------------------------------------------------------------------------------------------------------------------------------

if version eq 'v1.0' then begin

; Define general header
genheadlabel=[             'NUMBER OF GENERAL HEADER LINES (AFTER THIS LINE)']    & genhead=['']         & ngenheadformat=1
genheadlabel=[genheadlabel,'TOLNET STANDARDIZED FORMAT VERSION FOR PROFILE DATA'] & genhead=[genhead,''] & ngenheadformat=ngenheadformat+1
genheadlabel=[genheadlabel,'NUMBER OF PROFILES IN THIS FILE']                     & genhead=[genhead,''] & ngenheadformat=ngenheadformat+1
genheadlabel=[genheadlabel,'NUMBER OF DATA COLUMNS FOR ALL PROFILES']             & genhead=[genhead,''] & ngenheadformat=ngenheadformat+1
genheadlabel=[genheadlabel,'COLUMN 1' ] & genhead=[genhead,'ALT, m, Altitude above sea level (center of sampling bin)                '] & columnformat='f0.1'                & missval=         -9999  & ndatacol=1
genheadlabel=[genheadlabel,'COLUMN 2' ] & genhead=[genhead,'O3ND, molec.m-3, Ozone Number Density (measured )                        '] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 3' ] & genhead=[genhead,'O3NDUncert, molec.m-3, Ozone Number Density Combined Standard Uncertainty'] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 4' ] & genhead=[genhead,'O3NDResol, m, Ozone Number Density Standardized Vertical Resolution      '] & columnformat=[columnformat,'f0.1'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 5' ] & genhead=[genhead,'Precision, %, Measurement Precision                                      '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 6' ] & genhead=[genhead,'ChRange, #, Channel Range (1.0 to N.0, nearest-field to farthest-field)  '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 7' ] & genhead=[genhead,'O3MR, ppbv, Ozone Mixing Ratio (derived)                                 '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 8' ] & genhead=[genhead,'O3MRUncert, ppbv, Ozone Mixing Ratio Combined Standard Uncertainty       '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 9' ] & genhead=[genhead,'Press, hPa, Air Pressure used to derive Ozone Mixing Ratio               '] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 10'] & genhead=[genhead,'PressUncert, hPa, Air Pressure Standard Uncertainty                      '] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 11'] & genhead=[genhead,'Temp, K, Air Temperature used to derive Ozone Mixing Ratio               '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 12'] & genhead=[genhead,'TempUncert, K, Air Temperature Ratio Standard Uncertainty                '] & columnformat=[columnformat,'f0.2'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 13'] & genhead=[genhead,'AirND, molec.m-3, Air Number Density used to derive Ozone Mixing Ratio   '] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'COLUMN 14'] & genhead=[genhead,'AirNDUncert, molec.m-3, Air Number Density Standard Uncertainty          '] & columnformat=[columnformat,'e0.3'] & missval=[missval,-9999] & ndatacol=ndatacol+1
genheadlabel=[genheadlabel,'MISSING DATA VALUES'] & genhead=[genhead,strjoin(strtrim(missval,2),', ')] & ngenheadformat=ngenheadformat+1

; Define I/O format
ioformat=strarr(7)
ioformat(0)='(a,t8,a4,a)'
ioformat(1)='(a,t75,a4,a)'
ioformat(2)='(a,t'+strtrim(strlen(genhead(n_elements(genhead)-1))+1,2)+',a4,a)'
ioformat(3)='(a,t60,a4,a)'
ioformat(4)='(a,t48,a4,a)'
ioformat(5)='(a,a4,a)'
ioformat(6)='('+strjoin(columnformat,',", ",')+')'

; Define general comment section (excluding optional comments)
gencommlabel=[          'NUMBER OF GENERAL COMMENTS LINES (AFTER THIS LINE)'] & ngencommformat=1
gencommlabel=[gencommlabel,'INSTRUMENT NAME']                                       & ngencommmeta=1
gencommlabel=[gencommlabel,'PI AND CONTACT INFO']                                   & ngencommmeta=ngencommmeta+1
gencommlabel=[gencommlabel,'SITE NAME']                                             & ngencommmeta=ngencommmeta+1
gencommlabel=[gencommlabel,'SITE LONGITUDE, LATITUDE, ELEVATION (degE, degN, m)']   & ngencommmeta=ngencommmeta+1
gencommlabel=[gencommlabel,'DATA REVISION # (if value >0 then provide text below)'] & ngencommmeta=ngencommmeta+1

; Define record header (excluding optional comments)
reclabel=[         'NUMBER OF HEADER LINES IN THIS PROFILE"S HEADER (AFTER THIS LINE)'] & nrecformat=1
reclabel=[reclabel,'NUMBER OF DATA LINES IN THIS PROFILE']                              & nrecformat=nrecformat+1
reclabel=[reclabel,'DATA PROCESSING DATE, TIME']                                              & nrecmeta=1
reclabel=[reclabel,'DATA PROCESSING VERSION']                                                 & nrecmeta=nrecmeta+1
reclabel=[reclabel,'RESULTS QUALITY (NOMINAL, FAIR, POOR)']                                   & nrecmeta=nrecmeta+1
reclabel=[reclabel,'PROFILE DATE, TIME (UT) START']                                           & nrecmeta=nrecmeta+1
reclabel=[reclabel,'PROFILE DATE, TIME (UT) END']                                             & nrecmeta=nrecmeta+1
reclabel=[reclabel,'PROFILE DATE, TIME (UT) MEAN']                                            & nrecmeta=nrecmeta+1
reclabel=[reclabel,'SOURCE OF A PRIORI Press, Temp, AirND USED TO DERIVE OZONE MIXING RATIO'] & nrecmeta=nrecmeta+1
reclabel=[reclabel,'SOURCE DATE, TIME (UT)']                                                  & nrecmeta=nrecmeta+1
reclabel=[reclabel,'SOURCE LONGITUDE, LATITUDE, ELEVATION (degE, degN, m)']                   & nrecmeta=nrecmeta+1
reclabel=[reclabel,'OPERATOR COMMENTS']                                                       & nrecmeta=nrecmeta+1
datalabel=strarr(ndatacol) & igcol0=min(where(strpos(genheadlabel,'COLUMN 1') ge 0))
for igcol=igcol0,igcol0+ndatacol-1 do begin
  splitgenhead=strsplit(genhead(igcol),',',/extract,/regex) & datalabel(igcol-igcol0)=splitgenhead(0)
endfor
datalabel=strjoin(datalabel,', ') & nrecformat=nrecformat+1

endif

return
end